@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
-__version__   = '2.1.5'
+__version__   = '2.1.6'
 __author__    = "Avinash Kak (kak@purdue.edu)"
-__date__      = '2021-September-6'   
-__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.1.5.html'
+__date__      = '2021-September-11'   
+__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.1.6.html'
 __copyright__ = "(C) 2021 Avinash Kak. Python Software Foundation."
 
 
@@ -155,10 +155,10 @@ DATA PREDICTION:
     divided by the average of the predicted and the actual values:
 
                            100                     | P_t   -   GT_t |
-            sMAPE    =    -----  \SUM_{t=1}^n  ---------------------------
-                            n                    | P_t   +   GT_t | / 2.0
+            sMAPE    =    -----  \SUM_{t=1}^N  ---------------------------
+                            N                    | P_t   +   GT_t | / 2.0
 
-    where n is the number of predictions made in a file.  P_t is the value
+    where N is the number of predictions made in a file.  P_t is the value
     predicted by the network at inference time and GT_t is the ground-truth
     value that actually occurred at that time.  I have used the same metric
     for estimating the accuracy of the prediction results produced by pmGRU.
@@ -244,7 +244,7 @@ class DataPrediction(object):
             ## for forget gate:
             self.project1 = nn.Sequential( nn.Linear(self.input_size + self.hidden_size, self.hidden_size), nn.Sigmoid() )
             ## for interim out:
-            self.project2 = nn.Sequential( nn.Linear( self.input_size + self.hidden_size, self.hidden_size), nn.Tanh() )      
+            self.project2 = nn.Sequential( nn.Linear( self.input_size + self.hidden_size, self.hidden_size), nn.Tanh() ) 
             ## for final out
             self.project3 = nn.Sequential( nn.Linear( self.hidden_size, self.output_size ), nn.Tanh() )                   
     
@@ -291,7 +291,7 @@ class DataPrediction(object):
             df['dayofweek'] = df.apply(lambda x: x['Datetime'].dayofweek, axis=1)
             df['month'] = df.apply(lambda x: x['Datetime'].month, axis=1)
             df['dayofyear'] = df.apply(lambda x: x['Datetime'].dayofyear, axis=1)
-            ##  Now sort the rows based on the values in teh 'Datetime' column and then drop the 'Datetime' column:
+            ##  Now sort the rows based on the values in the 'Datetime' column and then drop the 'Datetime' column:
             df = df.sort_values("Datetime").drop("Datetime", axis=1)
             print("\n[construct dataframes]  Shape of the DataFrame object for file %s: " % file, df.shape)  ##  like: (32896, 5) 
             dataframes[file] = df
@@ -429,26 +429,29 @@ class DataPrediction(object):
         print("\n\nStarting Training")
         start_time = time.perf_counter()                                           
         epochs = self.dls.epochs
+        loss_tally = []
         for epoch in range(epochs):        
-            counter = 0                                                     
             avg_loss = 0.                                                   
             running_loss = 0.0
-            for x, label in train_loader:
-                label = label.to(device).float()
-                h = model.init_hidden()                                          
-                counter += 1                                                
-                for load_sample_index in range(x.shape[1]):
+            for counter,data in enumerate(train_loader):       
+                x,pred = data
+                optimizer.zero_grad()
+                ##  Shape of x:   (batch_size, sequence_length, input_size)
+                ##  For power-load dataset and for the constructor params chosen, the shape of x is  (1024,90,5)              
+                pred = pred.to(device).float()
+                h = model.init_hidden().data.to(device)
+                ##  The following loop steps through each element of the input sequence, one element at a
+                ##  time.  Note that x.shape[1] is the sequence_length, which is 90 as set by the constructor
+                ##  in the script in the ExamplesDataPrediction directory.
+                for load_sample_index in range(x.shape[1]):   
                     input_sample = x[:,load_sample_index,:]
                     input_sample = torch.squeeze(input_sample)
                     input_sample = input_sample.to(device).float()
-                    h = h.data                                              
-                    h = h.to(device)
-                    model.zero_grad()
                     if load_sample_index < x.shape[1] - 1:
                         out, h = model(input_sample, h)      
                     elif load_sample_index == x.shape[1] - 1:
                         out, h = model(input_sample, h, sequence_end=True)      
-                loss = criterion(out, label)
+                loss = criterion(out, pred)
                 loss.backward()
                 optimizer.step()
                 running_loss += loss.item()
@@ -456,9 +459,19 @@ class DataPrediction(object):
                     current_time = time.perf_counter()
                     elapsed_time = current_time - start_time                 
                     avg_loss = running_loss / float(200)
+                    loss_tally.append(avg_loss)
                     print("\n[epoch:%d/%d  iter=%4d  elapsed_time=%3d secs]     Avg Loss: %.8f" %  (epoch+1, epochs, counter+1, elapsed_time, avg_loss))
                     running_loss = 0.0
         self.save_model(model)
+        print("\nFinished Training\n")                                                                                               
+        plt.figure(figsize=(10,5))
+        plt.title("Loss")
+        plt.plot(loss_tally)
+        plt.xlabel("Training iterations over 5 epochs")
+        plt.ylabel("loss")
+        plt.legend()
+        plt.savefig("loss_vs_iterations_predictor_training.png")
+        plt.show()
         return model
     
 
@@ -480,25 +493,20 @@ class DataPrediction(object):
                 print("\n\n\n[eval] file: ", file)
                 ## We access the MinMaxScaler instance for "unscaling" the predictions:
                 scaler = self.predictions_scaling_params[file] 
-                minval = torch.zeros(1,1)
-                maxval = torch.zeros(1,1)
-                minval[0,0] = scaler.data_min_[0]
-                maxval[0,0] = scaler.data_max_[0]
-                minval = minval.to(device)
-                maxval = maxval.to(device)
+                minval, maxval = torch.zeros(1,1), torch.zeros(1,1)
+                minval[0,0], maxval[0,0] = scaler.data_min_[0], scaler.data_max_[0]
+                minval, maxval = minval.to(device), maxval.to(device)
                 testdata = TensorDataset(torch.from_numpy(self.test_sequences[file]), 
                                                                   torch.from_numpy(self.test_gt_predictions[file]))
                 testdata_loader = DataLoader(testdata, batch_size=self.dls.batch_size, drop_last=True)      
-                for x, gt_predictions in testdata_loader:
+                for x, gt_predictions in testdata_loader:      ## x is of shape (batch_size, sequence_length, input_size)
                     gt_predictions = gt_predictions.to(device)
                     gt_predictions_unscaled = ( gt_predictions*(maxval - minval) + minval ).reshape(-1)
                     all_gt_predictions +=  gt_predictions_unscaled.tolist()
-                    h = model.init_hidden()                                          
+                    h = model.init_hidden().data.to(device)
                     for load_sample_index in range(x.shape[1]):
                         input_sample = x[:,load_sample_index,:]
                         input_sample = torch.squeeze(input_sample)
-                        h = h.data                                              
-                        h = h.to(device)
                         if load_sample_index < x.shape[1] - 1:
                             out, h = model(input_sample.to(device).float(), h)      
                         elif load_sample_index == x.shape[1] - 1:
